
/*
 * moblin-app-installer -- Moblin garage client application.
 * Copyright (c) 2009, Intel Corporation.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Author: Robert Staudinger <robertx.staudinger@intel.com>
 */

#include "mai-category.h"
#include "config.h"

G_DEFINE_TYPE (MaiCategory, mai_category, G_TYPE_OBJECT)

#define GET_PRIVATE(o) \
  (G_TYPE_INSTANCE_GET_PRIVATE ((o), MAI_TYPE_CATEGORY, MaiCategoryPrivate))

enum
{
  PROP_0,

  PROP_NAME,
  PROP_ICON,
  PROP_IS_DEFAULT
};

typedef struct
{
  gchar        *name;
  gchar        *icon;
  gboolean      is_default;
} MaiCategoryPrivate;

static void
_get_property (GObject    *object,
               guint       property_id,
               GValue     *value,
               GParamSpec *pspec)
{
  switch (property_id) {
  case PROP_NAME:
    g_value_set_string (value,
                        mai_category_get_name (MAI_CATEGORY (object)));
    break;
  case PROP_ICON:
    g_value_set_string (value,
                        mai_category_get_icon (MAI_CATEGORY (object)));
    break;
  case PROP_IS_DEFAULT:
    g_value_set_boolean (value,
                         mai_category_is_default (MAI_CATEGORY (object)));
    break;
  default:
    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
  }
}

static void
_set_property (GObject      *object,
               guint         property_id,
               const GValue *value,
               GParamSpec   *pspec)
{
  MaiCategoryPrivate *priv = GET_PRIVATE (object);

  switch (property_id) {
  case PROP_NAME:
    priv->name = g_value_dup_string (value);
    break;
  case PROP_ICON:
    priv->icon = g_value_dup_string (value);
    break;
  case PROP_IS_DEFAULT:
    priv->is_default = g_value_get_boolean (value);
    break;
  default:
    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
  }
}

static void
_dispose (GObject *object)
{
  MaiCategoryPrivate *priv = GET_PRIVATE (object);

  if (priv->name)
    g_free (priv->name), priv->name = NULL;

  if (priv->icon)
    g_free (priv->icon), priv->icon = NULL;

  G_OBJECT_CLASS (mai_category_parent_class)->dispose (object);
}

static void
mai_category_class_init (MaiCategoryClass *klass)
{
  GObjectClass *object_class = G_OBJECT_CLASS (klass);
  GParamFlags param_flags;

  g_type_class_add_private (klass, sizeof (MaiCategoryPrivate));

  object_class->get_property = _get_property;
  object_class->set_property = _set_property;
  object_class->dispose = _dispose;

  param_flags = G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_NAME;

  g_object_class_install_property (object_class,
                                   PROP_NAME,
                                   g_param_spec_string ("name", "Name",
                                                        "Category name",
                                                        NULL, param_flags));
  g_object_class_install_property (object_class,
                                   PROP_ICON,
                                   g_param_spec_string ("icon", "Icon",
                                                        "Category icon",
                                                        NULL, param_flags));
  g_object_class_install_property (object_class,
                                   PROP_IS_DEFAULT,
                                   g_param_spec_boolean ("is-default",
                                                         "Default category",
                                                         "Whether category is default active",
                                                         FALSE, param_flags));
}

static void
mai_category_init (MaiCategory *self)
{
}

MaiCategory *
mai_category_new (const gchar  *name,
                  const gchar  *icon,
                  gboolean      is_default)
{
  g_return_val_if_fail (name, NULL);

  return g_object_new (MAI_TYPE_CATEGORY,
                       "name",        name,
                       "icon",        icon,
                       "is-default",  is_default,
                       NULL);
}

const gchar *
mai_category_get_name (MaiCategory const *self)
{
  g_return_val_if_fail (self, NULL);

  return GET_PRIVATE (self)->name;
}

const gchar *
mai_category_get_icon (MaiCategory const *self)
{
  g_return_val_if_fail (self, NULL);

  return GET_PRIVATE (self)->icon;
}

gboolean
mai_category_is_default (MaiCategory const  *self)
{
  g_return_val_if_fail (self, FALSE);

  return GET_PRIVATE (self)->is_default;
}

