
/*
 * moblin-app-installer -- Moblin garage client application.
 * Copyright (c) 2009, Intel Corporation.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Author: Robert Staudinger <robertx.staudinger@intel.com>
 */

#ifndef MAI_EXPANDER_H
#define MAI_EXPANDER_H

#include <glib-object.h>
#include <nbtk/nbtk.h>
#include "mai-package-status.h"

G_BEGIN_DECLS

#define MAI_TYPE_EXPANDER mai_expander_get_type()

#define MAI_EXPANDER(obj) \
  (G_TYPE_CHECK_INSTANCE_CAST ((obj), MAI_TYPE_EXPANDER, MaiExpander))

#define MAI_EXPANDER_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_CAST ((klass), MAI_TYPE_EXPANDER, MaiExpanderClass))

#define MAI_IS_EXPANDER(obj) \
  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), MAI_TYPE_EXPANDER))

#define MAI_IS_EXPANDER_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_TYPE ((klass), MAI_TYPE_EXPANDER))

#define MAI_EXPANDER_GET_CLASS(obj) \
  (G_TYPE_INSTANCE_GET_CLASS ((obj), MAI_TYPE_EXPANDER, MaiExpanderClass))

typedef struct
{
  NbtkBin parent;
} MaiExpander;

typedef struct
{
  NbtkBinClass parent_class;

  /* Signals */
  void (* cancel)           (MaiExpander *self);
  void (* install)          (MaiExpander *self);
  void (* remove)           (MaiExpander *self);
  void (* resolve)          (MaiExpander *self);

} MaiExpanderClass;

GType
mai_expander_get_type (void);

NbtkWidget *
mai_expander_new                  (void);

const gchar *
mai_expander_get_icon             (MaiExpander const  *self);

const gchar *
mai_expander_get_name             (MaiExpander const  *self);

const gchar *
mai_expander_get_description      (MaiExpander const  *self);

const gchar *
mai_expander_get_package_name     (MaiExpander const  *self);

MaiPackageStatus
mai_expander_get_status           (MaiExpander const  *self);

void
mai_expander_set_status           (MaiExpander        *self,
                                   MaiPackageStatus    status);

gint
mai_expander_get_progress         (MaiExpander const  *self);

void
mai_expander_set_progress         (MaiExpander        *self,
                                   gint                progress);

const gchar *
mai_expander_get_status_message   (MaiExpander const  *self);

void
mai_expander_set_status_message   (MaiExpander        *self,
                                   const gchar        *error_message);

const gchar *
mai_expander_get_long_description (MaiExpander const  *self);

gboolean
mai_expander_get_expanded         (MaiExpander        *self);

void
mai_expander_set_expanded         (MaiExpander        *self,
                                   gboolean            expanded);

G_END_DECLS

#endif /* MAI_EXPANDER_H */

