
/*
 * moblin-app-installer -- Moblin garage client application.
 * Copyright (c) 2009, Intel Corporation.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Author: Robert Staudinger <robertx.staudinger@intel.com>
 */

#include "mai-hyperlink-button.h"

G_DEFINE_TYPE (MaiHyperlinkButton, mai_hyperlink_button, NBTK_TYPE_BUTTON)

static void
_style_changed_cb (NbtkStylable *self,
                   gpointer      user_data)
{
  PangoAttrList *attributes = NULL;
  ClutterActor  *actor;

  /* Underlined label, except when hovered. */
  if (0 != g_strcmp0 ("hover", nbtk_stylable_get_pseudo_class (self)))
  {
    attributes = pango_attr_list_new ();
    pango_attr_list_insert (attributes,
                            pango_attr_underline_new (PANGO_UNDERLINE_SINGLE));
  }

  if (NULL != (actor = nbtk_bin_get_child (NBTK_BIN (self))) &&
      CLUTTER_IS_TEXT (actor))
  {
    clutter_text_set_attributes (CLUTTER_TEXT (actor), attributes);
  }

  if (attributes)
  {
    pango_attr_list_unref (attributes);
  }
}

static void
mai_hyperlink_button_class_init (MaiHyperlinkButtonClass *klass)
{
}

static void
mai_hyperlink_button_init (MaiHyperlinkButton *self)
{
  g_signal_connect (NBTK_STYLABLE (self), "style-changed",
                    G_CALLBACK (_style_changed_cb), NULL);
}

ClutterActor *
mai_hyperlink_button_new_with_label (const gchar *label)
{
  return g_object_new (MAI_TYPE_HYPERLINK_BUTTON,
                       "label", label,
                       NULL);
}

