
/*
 * moblin-app-installer -- Moblin garage client application.
 * Copyright (c) 2009, Intel Corporation.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Author: Robert Staudinger <robertx.staudinger@intel.com>
 */

#include "mai-list-view.h"

G_DEFINE_TYPE (MaiListView, mai_list_view, NBTK_TYPE_ITEM_VIEW)

typedef struct
{
  gfloat                  x1;
  gfloat                  y1;
  gfloat                  x2;
  gfloat                  gap;
  ClutterAllocationFlags  flags;
  gboolean                do_allocate;
} row_allocation_t;

static ClutterActorClass *mai_list_view_actor_parent_class = NULL;

static void
_allocate_row_cb (ClutterActor      *actor,
                  row_allocation_t  *row_allocation)
{
  ClutterActorBox box;
  gfloat          height;

  clutter_actor_get_preferred_height (actor,
                                      row_allocation->x2 - row_allocation->x1,
                                      NULL,
                                      &height);

  box.x1 = row_allocation->x1;
  box.y1 = row_allocation->y1;
  box.x2 = row_allocation->x2;
  box.y2 = box.y1 + height;

  if (row_allocation->do_allocate)
    clutter_actor_allocate (actor, &box, row_allocation->flags);

  row_allocation->y1 = box.y2 + row_allocation->gap;
}

#if 0
typedef struct
{
  gfloat  for_height;
  gfloat  min_width;
  gfloat  natural_width;
} preferred_width_t;

static void
_get_preferred_width_cb (ClutterActor       *actor,
                         preferred_width_t  *preferred_width)
{
  gfloat min_width;
  gfloat natural_width;

  clutter_actor_get_preferred_width (actor, 
                                     preferred_width->for_height,
                                     &min_width,
                                     &natural_width);

  if (min_width > preferred_width->min_width)
    preferred_width->min_width = min_width;

  if (natural_width > preferred_width->natural_width)
    preferred_width->natural_width = natural_width;
}

static void
_get_preferred_width (ClutterActor *self,
                      gfloat        for_height,
                      gfloat       *min_width_p,
                      gfloat       *natural_width_p)
{
  ClutterActor      *parent;
  NbtkPadding        padding;
  preferred_width_t  preferred_width = { 0, };

  nbtk_widget_get_padding (NBTK_WIDGET (self), &padding);

  parent = clutter_actor_get_parent (self);
  if (NBTK_IS_SCROLL_VIEW (parent))
  {
    if (min_width_p)
      *min_width_p = 1000;

    if (natural_width_p)
      *natural_width_p = 1000;

    return;
  }

  preferred_width.for_height = for_height;
  clutter_container_foreach (CLUTTER_CONTAINER (self),
                             (ClutterCallback) _get_preferred_width_cb,
                             &preferred_width);

  if (min_width_p)
    *min_width_p = preferred_width.min_width + padding.left + padding.right;

  if (natural_width_p)
    *natural_width_p = preferred_width.natural_width + padding.left + padding.right;
}
#endif

static void
_get_preferred_height (ClutterActor *self,
                       gfloat        for_width,
                       gfloat       *min_height_p,
                       gfloat       *natural_height_p)
{
  row_allocation_t  row_allocation;
  NbtkPadding       padding;

  nbtk_widget_get_padding (NBTK_WIDGET (self), &padding);

  row_allocation.x1     = padding.left;
  row_allocation.y1     = padding.top;
  if (for_width < 0)
    row_allocation.x2   = clutter_actor_get_width (self) - padding.right;
  else
    row_allocation.x2   = for_width - padding.right;
  row_allocation.gap    = nbtk_grid_get_row_gap (NBTK_GRID (self));
  row_allocation.do_allocate = TRUE;

  clutter_container_foreach (CLUTTER_CONTAINER (self),
                             (ClutterCallback) _allocate_row_cb,
                             &row_allocation);

  if (min_height_p)
    *min_height_p = row_allocation.y1 - row_allocation.gap + padding.bottom;

  if (natural_height_p)
    *natural_height_p = row_allocation.y1 - row_allocation.gap + padding.bottom;
}

static void
_allocate (ClutterActor           *self,
           const ClutterActorBox  *box,
           ClutterAllocationFlags  flags)
{
  NbtkAdjustment    *vadjust;
  NbtkAdjustment    *hadjust;
  row_allocation_t   row_allocation;
  NbtkPadding        padding;

  /* chain up here to preserve the allocated size
   *
   * (we ignore the height of the allocation if we have a vadjustment set,
   *  or the width of the allocation if an column_major is set and an
   *  hadjustment is set) */
  CLUTTER_ACTOR_CLASS (mai_list_view_actor_parent_class)->allocate (self, box, flags);

  nbtk_widget_get_padding (NBTK_WIDGET (self), &padding);

  row_allocation.x1     = padding.left;
  row_allocation.y1     = padding.top;
  row_allocation.x2     = box->x2 - box->x1 - padding.right;
  row_allocation.gap    = nbtk_grid_get_row_gap (NBTK_GRID (self));
  row_allocation.flags  = flags;
  row_allocation.do_allocate = TRUE;

  clutter_container_foreach (CLUTTER_CONTAINER (self),
                             (ClutterCallback) _allocate_row_cb,
                             &row_allocation);

  nbtk_scrollable_get_adjustments (NBTK_SCROLLABLE (self), &hadjust, &vadjust);
  if (vadjust)
  {
    gfloat height = row_allocation.y1 - row_allocation.gap + padding.bottom;
    g_object_set (G_OBJECT (vadjust),
                  "lower", 0.0,
                  "upper", height,
                  "page-size", box->y2 - box->y1,
                  "step-increment", (box->y2 - box->y1) / 6,
                  "page-increment", box->y2 - box->y1,
                  NULL);  
  }
  if (hadjust)
    {
      g_object_set (G_OBJECT (hadjust),
                    "lower", 0.0,
                    "upper", 0.0,
                    NULL);
    }
}

static void
mai_list_view_class_init (MaiListViewClass *klass)
{
  ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);

  /* actor_class->get_preferred_width = _get_preferred_width; */
  actor_class->get_preferred_height = _get_preferred_height;
  actor_class->allocate = _allocate;

  if (NULL == mai_list_view_actor_parent_class)
  {
    mai_list_view_actor_parent_class = g_type_class_peek (g_type_from_name ("ClutterActor"));
  }
}

static void
mai_list_view_init (MaiListView *self)
{
}

ClutterActor *
mai_list_view_new (void)
{
  return g_object_new (MAI_TYPE_LIST_VIEW,
                       "max-stride", 1,
                       NULL);
}

