
/*
 * moblin-app-installer -- Moblin garage client application.
 * Copyright (c) 2009, Intel Corporation.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Author: Robert Staudinger <robertx.staudinger@intel.com>
 */

#ifndef MAI_MODEL_H
#define MAI_MODEL_H

#include <glib-object.h>
#include "mai-category.h"
#include "mai-object-store.h"
#include "mai-package.h"

G_BEGIN_DECLS

#define MAI_TYPE_MODEL mai_model_get_type()

#define MAI_MODEL(obj) \
  (G_TYPE_CHECK_INSTANCE_CAST ((obj), MAI_TYPE_MODEL, MaiModel))

#define MAI_MODEL_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_CAST ((klass), MAI_TYPE_MODEL, MaiModelClass))

#define MAI_IS_MODEL(obj) \
  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), MAI_TYPE_MODEL))

#define MAI_IS_MODEL_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_TYPE ((klass), MAI_TYPE_MODEL))

#define MAI_MODEL_GET_CLASS(obj) \
  (G_TYPE_INSTANCE_GET_CLASS ((obj), MAI_TYPE_MODEL, MaiModelClass))

typedef struct
{
  GObject parent;
} MaiModel;

typedef struct
{
  GObjectClass parent;
} MaiModelClass;

enum
{
  MAI_MODEL_ERROR_PACKAGE_NOT_FOUND
};

GType
mai_model_get_type (void);

MaiModel *
mai_model_new                 (const gchar        *appstore_file);

const gchar *
mai_model_get_appstore_file   (MaiModel const     *self);

ClutterModel *
mai_model_get_store           (MaiModel const     *self);

GList const *
mai_model_get_categories      (MaiModel const     *self);

GList *
mai_model_get_used_categories (MaiModel const     *self);

void
mai_model_filter_category     (MaiModel           *self,
                               MaiCategory const  *category);
void
mai_model_filter_text         (MaiModel           *self,
                               const gchar        *text);
void
mai_model_filter_clear        (MaiModel           *self);

MaiPackage *
mai_model_lookup_package      (MaiModel           *self,
                               const gchar        *package_name);

G_END_DECLS

#endif /* MAI_MODEL_H */
