
/*
 * moblin-app-installer -- Moblin garage client application.
 * Copyright (c) 2009, Intel Corporation.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Author: Robert Staudinger <robertx.staudinger@intel.com>
 */

#include "mai-package-button.h"
#include "mai-shell-dimensions.h"

static void _stylable_iface_init (NbtkStylableIface *iface);

G_DEFINE_TYPE_WITH_CODE (MaiPackageButton, mai_package_button, NBTK_TYPE_BUTTON,
                         G_IMPLEMENT_INTERFACE (NBTK_TYPE_STYLABLE, _stylable_iface_init))

/*
 * NbtkStylable, needed to get parent type styling applied.
 */

static const gchar *
_stylable_get_style_type (NbtkStylable *stylable)
{
  return "NbtkButton";
}

static void
_stylable_iface_init (NbtkStylableIface *iface)
{
  static gboolean _is_initialized = FALSE;

  if (!_is_initialized)
  {
    iface->get_style_type = _stylable_get_style_type;
    _is_initialized = TRUE;
  }
}

/*
 * MaiPackageButton
 */

static void
_get_preferred_width (ClutterActor  *self,
                      gfloat         for_height,
                      gfloat        *min_width_p,
                      gfloat        *natural_width_p)
{
  CLUTTER_ACTOR_CLASS (mai_package_button_parent_class)
    ->get_preferred_width (self, for_height, min_width_p, natural_width_p);

  if (min_width_p && *min_width_p < MAI_PACKAGE_BUTTON_MIN_WIDTH)
    *min_width_p = MAI_PACKAGE_BUTTON_MIN_WIDTH;

  if (natural_width_p && *natural_width_p < MAI_PACKAGE_BUTTON_MIN_WIDTH)
    *natural_width_p = MAI_PACKAGE_BUTTON_MIN_WIDTH;
}

static void
mai_package_button_class_init (MaiPackageButtonClass *klass)
{
  ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);

  actor_class->get_preferred_width = _get_preferred_width;
}

static void
mai_package_button_init (MaiPackageButton *self)
{
}

ClutterActor *
mai_package_button_new_with_label (const gchar *label)
{
  return g_object_new (MAI_TYPE_PACKAGE_BUTTON,
                       "label", label,
                       NULL);
}

