
/*
 * moblin-app-installer -- Moblin garage client application.
 * Copyright (c) 2009, Intel Corporation.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Author: Robert Staudinger <robertx.staudinger@intel.com>
 */

#ifndef MAI_PACKAGE_H
#define MAI_PACKAGE_H

#include <glib-object.h>
#include "mai-package-status.h"

G_BEGIN_DECLS

#define MAI_TYPE_PACKAGE mai_package_get_type()

#define MAI_PACKAGE(obj) \
  (G_TYPE_CHECK_INSTANCE_CAST ((obj), MAI_TYPE_PACKAGE, MaiPackage))

#define MAI_PACKAGE_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_CAST ((klass), MAI_TYPE_PACKAGE, MaiPackageClass))

#define MAI_IS_PACKAGE(obj) \
  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), MAI_TYPE_PACKAGE))

#define MAI_IS_PACKAGE_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_TYPE ((klass), MAI_TYPE_PACKAGE))

#define MAI_PACKAGE_GET_CLASS(obj) \
  (G_TYPE_INSTANCE_GET_CLASS ((obj), MAI_TYPE_PACKAGE, MaiPackageClass))

typedef struct
{
  GObject parent;
} MaiPackage;

typedef struct
{
  GObjectClass parent;
} MaiPackageClass;

GType
mai_package_get_type (void);

enum
{
  MAI_PACKAGE_ERROR_OPERATION_UNAVAILABLE
};

MaiPackage *
mai_package_new                   (const gchar        *package,
                                   const gchar        *name,
                                   const gchar        *version,
                                   const gchar        *description,
                                   const gchar        *long_description,
                                   const gchar        *icon,
                                   const gchar       **categories);

gchar const *
mai_package_get_package_id        (MaiPackage         *self);

void
mai_package_set_package_id        (MaiPackage         *self,
                                   gchar const        *package_id);

const gchar *
mai_package_get_package_name      (MaiPackage const   *self);

const gchar *
mai_package_get_name              (MaiPackage const   *self);

const gchar *
mai_package_get_version           (MaiPackage const   *self);

const gchar *
mai_package_get_description       (MaiPackage const   *self);

const gchar *
mai_package_get_long_description  (MaiPackage const   *self);

const gchar *
mai_package_get_icon              (MaiPackage const   *self);

MaiPackageStatus
mai_package_get_status            (MaiPackage const   *self);

void
mai_package_set_status            (MaiPackage         *self,
                                   MaiPackageStatus    status);

const gchar **
mai_package_get_categories        (MaiPackage const   *self);

gint
mai_package_get_progress          (MaiPackage const   *self);

const gchar *
mai_package_get_message           (MaiPackage const   *self);

void
mai_package_set_message           (MaiPackage       *self,
                                   const gchar      *message);

gint
mai_package_compare_version    (MaiPackage const   *self,
                                const gchar        *version);

gboolean
mai_package_resolve               (MaiPackage         *self,
                                   GError            **error);

gboolean
mai_package_install               (MaiPackage         *self,
                                   GError            **error);

gboolean
mai_package_remove                (MaiPackage         *self,
                                   GError            **error);

G_END_DECLS

#endif /* MAI_PACKAGE_H */
