
/*
 * moblin-app-installer -- Moblin garage client application.
 * Copyright (c) 2009, Intel Corporation.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Author: Robert Staudinger <robertx.staudinger@intel.com>
 */

#ifndef MAI_PARSER_H
#define MAI_PARSER_H

#include <glib-object.h>
#include "mai-category.h"
#include "mai-package.h"

G_BEGIN_DECLS

#define MAI_TYPE_PARSER mai_parser_get_type()

#define MAI_PARSER(obj) \
  (G_TYPE_CHECK_INSTANCE_CAST ((obj), MAI_TYPE_PARSER, MaiParser))

#define MAI_PARSER_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_CAST ((klass), MAI_TYPE_PARSER, MaiParserClass))

#define MAI_IS_PARSER(obj) \
  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), MAI_TYPE_PARSER))

#define MAI_IS_PARSER_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_TYPE ((klass), MAI_TYPE_PARSER))

#define MAI_PARSER_GET_CLASS(obj) \
  (G_TYPE_INSTANCE_GET_CLASS ((obj), MAI_TYPE_MODEL, MaiParserClass))

typedef struct
{
  GObject parent;
} MaiParser;

typedef struct
{
  GObjectClass parent;
} MaiParserClass;

typedef struct MaiParserIterator_ MaiParserIterator;

GType
mai_parser_get_type (void);

MaiParser *
mai_parser_new                    (const gchar      *appstore_file);

const gchar *
mai_parser_get_appstore_file      (MaiParser const  *self);

MaiParserIterator *
mai_parser_get_category_iterator  (MaiParser const  *self);

MaiParserIterator *
mai_parser_get_package_iterator   (MaiParser const  *self);

/* Iterator */

gboolean
mai_parser_iterator_next            (MaiParserIterator        *iter);

MaiCategory *
mai_parser_iterator_create_category (MaiParserIterator const  *iter);

MaiPackage *
mai_parser_iterator_create_package  (MaiParserIterator const  *iter);

gboolean
mai_parser_iterator_is_end          (MaiParserIterator const  *iter);

void
mai_parser_iterator_destroy         (MaiParserIterator        *iter);

G_END_DECLS

#endif /* MAI_PARSER_H */

