
/*
 * moblin-app-installer -- Moblin garage client application.
 * Copyright (c) 2009, Intel Corporation.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Author: Robert Staudinger <robertx.staudinger@intel.com>
 * Based on code by Shane Bryan <shane.bryan@linux.intel.com>
 */

#include <string.h>
#include <glib/gi18n.h>
#include <moblin-panel/mpl-entry.h>
#include "mai-category-button.h"
#include "mai-expander.h"
#include "mai-hyperlink-button.h"
#include "mai-list-view.h"
#include "mai-shell.h"
#include "mai-shell-dimensions.h"
#include "mai-utils.h"
#include "config.h"

#define SEARCH_APPLY_TIMEOUT 500

G_DEFINE_TYPE (MaiShell, mai_shell, NBTK_TYPE_TABLE)

#define GET_PRIVATE(o) \
  (G_TYPE_INSTANCE_GET_PRIVATE ((o), MAI_TYPE_SHELL, MaiShellPrivate))

enum
{
  PROP_0,

  PROP_MODEL
};

enum
{
  SHOW_URI,
  QUIT_REQUEST,
  CANCEL,
  INSTALL,
  REMOVE,
  RESOLVE,

  LAST_SIGNAL
};

typedef struct {
  /* Properties */
  MaiModel      *model;

  /* Managed by clutter */
  ClutterActor  *titlebar;
  ClutterActor  *filter_entry;
  ClutterActor  *category_list;
  ClutterActor  *application_pane;
  ClutterActor  *application_list;
  ClutterActor  *statusbar;

  /* Data */
  guint          filter_entry_text_changed_id;
  guint          timeout_id;
} MaiShellPrivate;

static void
mai_shell_set_model (MaiShell *self,
                     MaiModel *model);

static guint _signals[LAST_SIGNAL] = { 0, };

static void
_quit_clicked_cb (NbtkButton  *button,
                  MaiShell    *self)
{
  g_signal_emit (self, _signals[QUIT_REQUEST], 0, NULL);
}

static void
_uncheck_other_buttons (NbtkButton *button,
                        NbtkButton *checked_button)
{
  if (button != checked_button)
    nbtk_button_set_checked (button, FALSE);
}

static void
_contract_other_expanders (MaiExpander *expander,
                           MaiExpander *expanded_expander)
{
  if (expander != expanded_expander)
    mai_expander_set_expanded (expander, FALSE);
}

static void
_get_first_child (ClutterActor  *actor,
                  ClutterActor  **first_child)
{
  if (*first_child == NULL)
    *first_child = actor;
}

static void
_expander_cancel_cb (MaiExpander *expander,
                     MaiShell    *self)
{
  g_signal_emit_by_name (self, "cancel",
                         mai_expander_get_package_name (expander));
/*
  MaiShellPrivate *priv = GET_PRIVATE (self);
  GError          *error = NULL;

  mai_model_cancel_transaction (priv->model,
                                mai_expander_get_package_name (expander),
                                &error);
  if (error)
  {
    mai_expander_set_status (expander, MAI_PACKAGE_STATUS_ERROR);
    mai_expander_set_status_message (expander, error->message);
    g_warning ("%s : %s", G_STRLOC, error->message);
    g_clear_error (&error);
  }
*/
}

static void
_expander_install_cb (MaiExpander *expander,
                      MaiShell    *self)
{
  g_signal_emit_by_name (self, "install",
                         mai_expander_get_package_name (expander));

/*
  MaiShellPrivate *priv = GET_PRIVATE (self);
  GError          *error = NULL;

  mai_model_install_package (priv->model,
                             mai_expander_get_package_name (expander),
                             &error);
  if (error)
  {
    mai_expander_set_status (expander, MAI_PACKAGE_STATUS_ERROR);
    mai_expander_set_status_message (expander, error->message);
    g_warning ("%s : %s", G_STRLOC, error->message);
    g_clear_error (&error);
  }
*/
}

static void
_expander_remove_cb (MaiExpander *expander,
                     MaiShell    *self)
{
  g_signal_emit_by_name (self, "remove",
                         mai_expander_get_package_name (expander));

/*
  MaiShellPrivate *priv = GET_PRIVATE (self);
  GError          *error = NULL;

  mai_model_remove_package (priv->model,
                            mai_expander_get_package_name (expander),
                            &error);
  if (error)
  {
    mai_expander_set_status (expander, MAI_PACKAGE_STATUS_ERROR);
    mai_expander_set_status_message (expander, error->message);
    g_warning ("%s : %s", G_STRLOC, error->message);
    g_clear_error (&error);
  }
*/
}

static void
_expander_resolve_cb (MaiExpander *expander,
                      MaiShell    *self)
{
  g_signal_emit_by_name (self, "resolve",
                         mai_expander_get_package_name (expander));
/*
  MaiShellPrivate *priv = GET_PRIVATE (self);
  GError          *error = NULL;

  mai_model_resolve_package (priv->model,
                             mai_expander_get_package_name (expander),
                             &error);
  if (error)
  {
    mai_expander_set_status (expander, MAI_PACKAGE_STATUS_ERROR);
    mai_expander_set_status_message (expander, error->message);
    g_warning ("%s : %s", G_STRLOC, error->message);
    g_clear_error (&error);
  }
*/
}

static void
_expander_expanded_notify_cb (MaiExpander *expander,
                              GParamSpec  *pspec,
                              MaiShell    *self)
{
  MaiShellPrivate *priv = GET_PRIVATE (self);

  if (mai_expander_get_expanded (expander))
  {
    clutter_container_foreach (CLUTTER_CONTAINER (priv->application_list),
                               (ClutterCallback) _contract_other_expanders,
                               expander);
  }
}

static void
_setup_expander (MaiExpander  *expander,
                 MaiShell     *self)
{
  gulong signal_id;

  signal_id = g_signal_handler_find (expander,
                                     G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA,
                                     0, 0, NULL,
                                     MAI_CALLBACK_TO_POINTER (_expander_install_cb),
                                     self);
  if (signal_id == 0)
  {
    /* Not set up yet. */
    g_signal_connect (G_OBJECT (expander), "cancel",
                      G_CALLBACK (_expander_cancel_cb), self);
    g_signal_connect (G_OBJECT (expander), "install",
                      G_CALLBACK (_expander_install_cb), self);
    g_signal_connect (G_OBJECT (expander), "remove",
                      G_CALLBACK (_expander_remove_cb), self);
    g_signal_connect (G_OBJECT (expander), "resolve",
                      G_CALLBACK (_expander_resolve_cb), self);
    g_signal_connect (G_OBJECT (expander), "notify::expanded",
                      G_CALLBACK (_expander_expanded_notify_cb), self);
  }
}

static void
_category_checked_cb (MaiCategoryButton *button,
                      GParamSpec        *pspec,
                      MaiShell          *self)
{
  MaiShellPrivate *priv = GET_PRIVATE (self);

  if (nbtk_button_get_checked (NBTK_BUTTON (button)))
  {
    /* Clear filter. */
    g_signal_handler_block (priv->filter_entry,
                            priv->filter_entry_text_changed_id);
    mpl_entry_set_text (MPL_ENTRY (priv->filter_entry), "");
    g_signal_handler_unblock (priv->filter_entry,
                              priv->filter_entry_text_changed_id);

    /* Uncheck other categories. */
    clutter_container_foreach (CLUTTER_CONTAINER (priv->category_list),
                               (ClutterCallback) _uncheck_other_buttons,
                               button);

    /* Filter */
    mai_model_filter_category (priv->model,
                               mai_category_button_get_category (button));

    /* Set up list-view. */
    clutter_container_foreach (CLUTTER_CONTAINER (priv->application_list),
                               (ClutterCallback) _setup_expander,
                               self);
  }
}

static gboolean
_filter_apply_cb (MaiShell *self)
{
  MaiShellPrivate *priv = GET_PRIVATE (self);
  const gchar     *filter_text;

  filter_text = mpl_entry_get_text (MPL_ENTRY (priv->filter_entry));

  if (filter_text && strlen (filter_text))
  {
    /* Uncheck categories. */
    clutter_container_foreach (CLUTTER_CONTAINER (priv->category_list),
                               (ClutterCallback) _uncheck_other_buttons,
                               NULL);
    /* Filter */
    mai_model_filter_text (priv->model, filter_text);

    /* Set up list-view. */
    clutter_container_foreach (CLUTTER_CONTAINER (priv->application_list),
                               (ClutterCallback) _setup_expander,
                               self);

  } else {
    /* Highlight first category. */
    ClutterActor *first_child = NULL;
    clutter_container_foreach (CLUTTER_CONTAINER (priv->category_list),
                               (ClutterCallback) _get_first_child,
                               &first_child);
    if (NBTK_IS_BUTTON (first_child))
      nbtk_button_set_checked (NBTK_BUTTON (first_child), TRUE);
  }

  return FALSE;
}

static void
_filter_entry_changed_cb (MplEntry  *entry,
                          MaiShell  *self)
{
  MaiShellPrivate *priv = GET_PRIVATE (self);

  /* Abort current search timout, if any. */
  if (priv->timeout_id)
  {
    g_source_remove (priv->timeout_id);
    priv->timeout_id = 0;
  }

  priv->timeout_id = g_timeout_add (SEARCH_APPLY_TIMEOUT,
                                    (GSourceFunc) _filter_apply_cb,
                                    self);
}

static void
_source_clicked_cb (NbtkButton  *button,
                    MaiShell    *self)
{
/* FIXME: keep up with the releases, possibly by parsing garage.repo.
 * http://repo.moblin.org/moblin/garage/2.0/oss/source/
 * http://repo.moblin.org/moblin/garage/2.1/oss/source/
 * etc.
 */

  g_signal_emit (self, _signals[SHOW_URI], 0,
                 "http://repo.moblin.org/moblin/garage/2.0/oss/source/");
}


static void
_get_property (GObject    *object,
               guint       property_id,
               GValue     *value,
               GParamSpec *pspec)
{
  switch (property_id)
  {
  case PROP_MODEL:
    g_value_set_object (value, mai_shell_get_model (MAI_SHELL (object)));
    break;
  default:
    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
  }
}

static void
_set_property (GObject      *object,
               guint         property_id,
               const GValue *value,
               GParamSpec   *pspec)
{
  switch (property_id)
  {
  case PROP_MODEL:
    mai_shell_set_model (MAI_SHELL (object),
                         MAI_MODEL (g_value_get_object (value)));
    break;
  default:
    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
  }
}

static void
_dispose (GObject *object)
{
  MaiShellPrivate *priv = GET_PRIVATE (object);

  if (priv->model)
    g_object_unref (priv->model), priv->model = NULL;

  G_OBJECT_CLASS (mai_shell_parent_class)->dispose (object);
}

static void
mai_shell_class_init (MaiShellClass *klass)
{
  GObjectClass *object_class = G_OBJECT_CLASS (klass);
  GParamFlags param_flags;

  g_type_class_add_private (klass, sizeof (MaiShellPrivate));

  object_class->get_property = _get_property;
  object_class->set_property = _set_property;
  object_class->dispose = _dispose;

  param_flags = G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_NAME;

  g_object_class_install_property (object_class,
                                   PROP_MODEL,
                                   g_param_spec_object ("model", "Model",
                                                        "Application backend",
                                                        MAI_TYPE_MODEL,
                                                        param_flags));

  _signals[SHOW_URI] = g_signal_new ("show-uri",
                                     G_TYPE_FROM_CLASS (object_class),
                                     G_SIGNAL_RUN_LAST, 0, NULL, NULL,
                                     g_cclosure_marshal_VOID__STRING,
                                     G_TYPE_NONE, 1, G_TYPE_STRING);

  _signals[QUIT_REQUEST] = g_signal_new ("quit-request",
                                         G_TYPE_FROM_CLASS (object_class),
                                         G_SIGNAL_RUN_LAST, 0, NULL, NULL,
                                         g_cclosure_marshal_VOID__VOID,
                                         G_TYPE_NONE, 0);

  _signals[CANCEL] = g_signal_new ("cancel",
                                   G_TYPE_FROM_CLASS (klass),
                                   G_SIGNAL_RUN_LAST, 0, NULL, NULL,
                                   g_cclosure_marshal_VOID__STRING,
                                   G_TYPE_NONE, 1, G_TYPE_STRING);

  _signals[INSTALL] = g_signal_new ("install",
                                    G_TYPE_FROM_CLASS (klass),
                                    G_SIGNAL_RUN_LAST, 0, NULL, NULL,
                                    g_cclosure_marshal_VOID__STRING,
                                    G_TYPE_NONE, 1, G_TYPE_STRING);

  _signals[REMOVE] = g_signal_new ("remove",
                                   G_TYPE_FROM_CLASS (klass),
                                   G_SIGNAL_RUN_LAST, 0, NULL, NULL,
                                   g_cclosure_marshal_VOID__STRING,
                                   G_TYPE_NONE, 1, G_TYPE_STRING);

  _signals[RESOLVE] = g_signal_new ("resolve",
                                    G_TYPE_FROM_CLASS (klass),
                                    G_SIGNAL_RUN_LAST, 0, NULL, NULL,
                                    g_cclosure_marshal_VOID__STRING,
                                    G_TYPE_NONE, 1, G_TYPE_STRING);
}

static ClutterActor *
create_application_pane (MaiShell *self)
{
  MaiShellPrivate *priv = GET_PRIVATE (self);
  ClutterActor *scroll = CLUTTER_ACTOR (nbtk_scroll_view_new ());

  priv->application_list = mai_list_view_new ();

  nbtk_item_view_set_item_type (NBTK_ITEM_VIEW (priv->application_list),
                                MAI_TYPE_EXPANDER);
  nbtk_item_view_add_attribute (NBTK_ITEM_VIEW (priv->application_list),
                                "package-name",     1);
  nbtk_item_view_add_attribute (NBTK_ITEM_VIEW (priv->application_list),
                                "name",             2);
  nbtk_item_view_add_attribute (NBTK_ITEM_VIEW (priv->application_list),
                                "description",      4);
  nbtk_item_view_add_attribute (NBTK_ITEM_VIEW (priv->application_list),
                                "long-description", 5);
  nbtk_item_view_add_attribute (NBTK_ITEM_VIEW (priv->application_list),
                                "icon",             6);
  nbtk_item_view_add_attribute (NBTK_ITEM_VIEW (priv->application_list),
                                "status",           8);
  nbtk_item_view_add_attribute (NBTK_ITEM_VIEW (priv->application_list),
                                "progress",         9);
  nbtk_item_view_add_attribute (NBTK_ITEM_VIEW (priv->application_list),
                                "status-message",  10);

  nbtk_widget_set_style_class_name (NBTK_WIDGET (priv->application_list),
                                    "MaiApplicationList");

  clutter_container_add_actor (CLUTTER_CONTAINER (scroll),
                               CLUTTER_ACTOR (priv->application_list));

  return scroll;
}

static gboolean
_set_key_focus (MaiShell *self)
{
  MaiShellPrivate *priv = GET_PRIVATE (self);

  clutter_actor_grab_key_focus (priv->filter_entry);

  return FALSE;
}

/*
 * Always have the application list take the available space.
 */
static void
_width_notify_cb (MaiShell    *self,
                  GParamSpec  *pspec,
                  gpointer     user_data)
{
  MaiShellPrivate *priv = GET_PRIVATE (self);
  NbtkPadding padding;
  gfloat      width;

  width = clutter_actor_get_width (CLUTTER_ACTOR (self));

  width -= clutter_actor_get_width (priv->category_list);

  width -= nbtk_table_get_col_spacing (NBTK_TABLE (self));

  nbtk_widget_get_padding (NBTK_WIDGET (self), &padding);
  width -= (padding.left + padding.right);

  clutter_actor_set_width (priv->application_list, width);
}

static void
mai_shell_init (MaiShell *self)
{
  MaiShellPrivate *priv = GET_PRIVATE (self);
  ClutterActor    *actor;

  g_signal_connect (self, "notify::width",
                    G_CALLBACK (_width_notify_cb), NULL);

  priv->titlebar = CLUTTER_ACTOR (g_object_new (NBTK_TYPE_TABLE,
                                              "row-spacing", 0,
                                              "col-spacing", 0,
                                              "homogeneous", FALSE,
                                               NULL));
  nbtk_widget_set_style_class_name (NBTK_WIDGET (priv->titlebar),
                                    "MaiTitleBar");

  /* Filter entry */
  priv->filter_entry = CLUTTER_ACTOR (mpl_entry_new (_("Search")));
  clutter_actor_set_width (priv->filter_entry, MAI_TITLE_FILTER_WIDTH);
  g_idle_add ((GSourceFunc) _set_key_focus, self);
  nbtk_table_add_actor_with_properties (NBTK_TABLE (priv->titlebar),
                                        priv->filter_entry,
                                        0, 0,
                                        "x-fill",   FALSE,
                                        "y-fill",   FALSE,
                                        "x-expand", FALSE,
                                        "y-expand", FALSE,
                                        "x-align",  0.0,
                                        "y-align",  0.5,
                                        NULL);
  priv->filter_entry_text_changed_id = g_signal_connect (priv->filter_entry,
                                                   "text-changed",
                                                   G_CALLBACK (_filter_entry_changed_cb),
                                                   self);

  /* Spacer */
  actor = CLUTTER_ACTOR (nbtk_bin_new ());
  nbtk_table_add_actor_with_properties (NBTK_TABLE (priv->titlebar),
                                        actor,
                                        0, 1,
                                        "x-fill",   TRUE,
                                        "y-fill",   FALSE,
                                        "x-expand", TRUE,
                                        "y-expand", FALSE,
                                        "x-align",  0.5,
                                        "y-align",  0.5,
                                        NULL);

  /* "Close" button. */
  actor = CLUTTER_ACTOR (nbtk_button_new ());
  clutter_actor_set_size (actor, MAI_TITLE_CLOSE_WIDTH, MAI_TITLE_CLOSE_HEIGHT);
  g_signal_connect (actor, "clicked",
                    G_CALLBACK (_quit_clicked_cb), self);
  nbtk_table_add_actor_with_properties (NBTK_TABLE (priv->titlebar),
                                        actor,
                                        0, 2,
                                        "x-fill",   FALSE,
                                        "y-fill",   FALSE,
                                        "x-expand", FALSE,
                                        "y-expand", FALSE,
                                        "x-align",  1.0,
                                        "y-align",  0.5,
                                        NULL);

  /* Title bar */
  nbtk_table_add_actor_with_properties (NBTK_TABLE (self), priv->titlebar,
                                        0, 0,
                                        "x-fill",   TRUE,
                                        "y-fill",   FALSE,
                                        "x-expand", TRUE,
                                        "y-expand", FALSE,
                                        "x-align",  0.0,
                                        "y-align",  0.0,
                                        "col-span", 2,
                                        NULL);

  /* List of package groups */
  priv->category_list = mai_list_view_new ();
  nbtk_widget_set_style_class_name (NBTK_WIDGET (priv->category_list),
                                    "MaiCategoryList");
  nbtk_table_add_actor_with_properties (NBTK_TABLE (self), priv->category_list,
                                        1, 0,
                                        "x-fill",   FALSE,
                                        "y-fill",   TRUE,
                                        "x-expand", FALSE,
                                        "y-expand", TRUE,
                                        "x-align",  0.0,
                                        "y-align",  0.0,
                                        NULL);

  /* List of packages in selected group */
  priv->application_pane = create_application_pane (self);
  nbtk_table_add_actor_with_properties (NBTK_TABLE (self), priv->application_pane,
                                        1, 1,
                                        "x-fill",   TRUE,
                                        "y-fill",   TRUE,
                                        "x-expand", TRUE,
                                        "y-expand", TRUE,
                                        "x-align",  0.0,
                                        "y-align",  0.0,
                                        "row-span", 2,
                                        NULL);

  /* "Get the source" */
  actor = mai_hyperlink_button_new_with_label (_("Get the source code"));
  g_signal_connect (actor, "clicked",
                    G_CALLBACK (_source_clicked_cb), self);
  nbtk_table_add_actor_with_properties (NBTK_TABLE (self), actor,
                                        2, 0,
                                        "x-fill",   TRUE,
                                        "y-fill",   FALSE,
                                        "x-expand", FALSE,
                                        "y-expand", FALSE,
                                        "x-align",  0.0,
                                        "y-align",  0.0,
                                        NULL);

  /* Status bar */
/* FIXME not done yet, hide away.
  priv->statusbar = CLUTTER_ACTOR (nbtk_label_new ("<status>"));
  nbtk_table_add_actor_with_properties (NBTK_TABLE (self),
                                        priv->statusbar,
                                        3, 0,
                                        "x-fill",   TRUE,
                                        "y-fill",   FALSE,
                                        "x-expand", FALSE,
                                        "y-expand", FALSE,
                                        "x-align",  0.0,
                                        "y-align",  0.5,
                                        "col-span", 2,
                                        NULL);
*/
}

ClutterActor *
mai_shell_new (MaiModel *model)
{
  return g_object_new (MAI_TYPE_SHELL,
                       "model", model,
                       NULL);
}

MaiModel *
mai_shell_get_model (MaiShell const *self)
{
  g_return_val_if_fail (MAI_IS_SHELL (self), NULL);

  return GET_PRIVATE (self)->model;
}

static void
mai_shell_set_model (MaiShell *self,
                     MaiModel *model)
{
  MaiShellPrivate *priv;
  GList           *categories;

  g_return_if_fail (MAI_IS_SHELL (self));
  g_return_if_fail (MAI_IS_MODEL (model));

  priv = GET_PRIVATE (self);

  /* Can only be set once. */
  g_return_if_fail (priv->model == NULL);

  priv->model = g_object_ref (model);

  nbtk_item_view_set_model (NBTK_ITEM_VIEW (priv->application_list),
                            mai_model_get_store (priv->model));

  categories = mai_model_get_used_categories (model);
  while (categories)
  {
    MaiCategory const *category = MAI_CATEGORY (categories->data);
    ClutterActor *button = mai_category_button_new (category);
    nbtk_widget_set_style_class_name (NBTK_WIDGET (button), "MaiCategoryButton");
#if defined (HAVE_NBTK_1_2)
    nbtk_bin_set_alignment (NBTK_BIN (button),
                            NBTK_ALIGN_START, NBTK_ALIGN_MIDDLE);
#elif defined (HAVE_NBTK_1_0)
    nbtk_bin_set_alignment (NBTK_BIN (button),
                            NBTK_ALIGN_LEFT, NBTK_ALIGN_CENTER);
#else
  /* configure should have caught this already. */
  #warning Unknown nbtk version
#endif
    clutter_container_add_actor (CLUTTER_CONTAINER (priv->category_list),
                                  CLUTTER_ACTOR (button));

    g_signal_connect (button, "notify::checked",
                      G_CALLBACK (_category_checked_cb), self);

    /* Have to defer this until the signal handler is set up. */
    nbtk_button_set_checked (NBTK_BUTTON (button),
                             mai_category_is_default (category));

    categories = g_list_delete_link (categories, categories);
  }

  g_object_notify (G_OBJECT (self), "model");
}

