
/*
 * moblin-app-installer -- Moblin garage client application.
 * Copyright (c) 2009, Intel Corporation.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Author: Robert Staudinger <robertx.staudinger@intel.com>
 * Based on code by Shane Bryan <shane.bryan@linux.intel.com>
 */

#include <locale.h>
#include <stdlib.h>

#include <glib/gi18n.h>
#include <clutter-gtk/clutter-gtk.h>
#include <gtk/gtk.h>
#include <nbtk/nbtk.h>

#include "mai-application.h"
#include "config.h"

#define MAI_CSS_FILE THEMEDIR "/theme.css"
#define MPL_CSS_FILE THEMEDIR "/moblin-panel/theme.css"
#define APP_LIST PKGDATADIR "/applist.xml"

int
main (int argc, char *argv[])
{
  const gchar *geometry = NULL;
  GOptionEntry options[] =
  {
    { "geometry",   'g', 0, G_OPTION_ARG_STRING, &geometry,   "Window geometry when standalone", NULL },
    { NULL }
  };

  NbtkStyle       *style;
  MaiApplication  *application;
  GError          *error = NULL;

  setlocale (LC_ALL, "");
  bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
  textdomain (GETTEXT_PACKAGE);

  gtk_clutter_init_with_args (&argc, &argv, "- Moblin app installer",
                              options, GETTEXT_PACKAGE, &error);
  if (error)
  {
    g_critical ("%s : %s", G_STRLOC, error->message);
    g_clear_error (&error);
  }

  /* Load any custom Nbtk Styles */
  style = nbtk_style_get_default ();
  if (!nbtk_style_load_from_file (style, MAI_CSS_FILE, &error))
  {
    g_warning ("Unable to load custom style (%s): %s",
               MAI_CSS_FILE,
               error->message);
    g_clear_error (&error);
  }
  if (!nbtk_style_load_from_file (style, MPL_CSS_FILE, &error))
  {
    g_warning ("Unable to load custom style (%s): %s",
               MPL_CSS_FILE,
               error->message);
    g_clear_error (&error);
  }

  application = g_object_new (MAI_TYPE_APPLICATION,
                              "window-geometry",  geometry,
                              "app-list",         APP_LIST,
                              NULL);
  mai_application_run (application);
  g_object_unref (application);

  return EXIT_SUCCESS;
}

