/*
 * Copyright 2009 Intel Corporation.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU Lesser General Public License,
 * version 2.1, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
 * more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 * Boston, MA 02111-1307, USA.
 *
 */
#include <stdio.h>
#include <stdlib.h>

#include <clutter/clutter.h>
#include "mai-expander.h"

#define MAI_CSS_FILE THEMEDIR "/theme.css"
#define ICON ABS_SRCDIR "/system-software-install.png"

static void
set_expanded (ClutterActor    *actor,
              ClutterKeyEvent *event,
              MaiExpander    *expander)
{
  gboolean expand;

  if (event->keyval != 32)
    return;

  expand = mai_expander_get_expanded (expander);

  mai_expander_set_expanded (expander, !expand);
}

static void
stage_size_notify_cb (ClutterActor *stage,
                      GParamSpec *pspec,
                      ClutterActor *table)
{
  gfloat width, height;

  clutter_actor_get_size (stage, &width, &height);
}

int
main (int argc, char *argv[])
{
  NbtkStyle       *style;
  GError          *error = NULL;
  NbtkWidget *expander;
  ClutterActor *stage;

  clutter_init (&argc, &argv);

  /* Load any custom Nbtk Styles */
  style = nbtk_style_get_default ();
  if (!nbtk_style_load_from_file (style, MAI_CSS_FILE, &error))
  {
    g_warning ("Unable to load custom style (%s): %s",
               MAI_CSS_FILE,
               error->message);
    g_clear_error (&error);
  }

  stage = clutter_stage_get_default ();
  clutter_actor_set_size (stage, 640, 480);
  clutter_stage_set_user_resizable (CLUTTER_STAGE (stage), TRUE);

  expander = g_object_new (MAI_TYPE_EXPANDER,
                           "icon", ICON,
                           "name", "Expander",
                           "description", "long long long long long long long "
                                          "description description description",
                           NULL);
  clutter_container_add_actor (CLUTTER_CONTAINER (stage),
                               CLUTTER_ACTOR (expander));
  clutter_actor_set_position (CLUTTER_ACTOR (expander), 10, 10);

  g_signal_connect (stage, "notify::width",
                    G_CALLBACK (stage_size_notify_cb), expander);
  g_signal_connect (stage, "notify::height",
                    G_CALLBACK (stage_size_notify_cb), expander);
  g_signal_connect (stage, "key-release-event",
                    G_CALLBACK (set_expanded), expander);
  clutter_actor_show (stage);
  clutter_main ();

  return EXIT_SUCCESS;
}
