
#include <config.h>

#include <stdio.h>
#include <stdlib.h>

#ifdef HAVE_PACKAGEKIT_GLIB2
 #define I_KNOW_THE_PACKAGEKIT_GLIB2_API_IS_SUBJECT_TO_CHANGE
 #include <packagekit-glib2/packagekit.h>
#else
 #include <packagekit-glib/packagekit.h>
#endif

#include <clutter/clutter.h>
#include <gtk/gtk.h>
#include "mai-object-store.h"
#include "mai-package.h"
#include "mai-utils.h"

/* Just so stuff links. */
PkControl *
mai_model_get_control (void);
PkControl *
mai_model_get_control (void)
{
  return NULL;
}

static void
dump_row (ClutterModel      *model,
          ClutterModelIter  *iter)
{
  guint   n_columns;
  GValue  value = { 0, };

  n_columns = clutter_model_get_n_columns (model);

  for (guint i = 1; i < n_columns; i++)
  {
    clutter_model_iter_get_value (iter, i, &value);
    printf ("%s, ", g_value_get_string (&value));
    g_value_unset (&value);
  }
  printf ("\n");
}

static void
dump (ClutterModel *model)
{
  ClutterModelIter *iter;

  for (iter = clutter_model_get_first_iter (model);
       iter && !clutter_model_iter_is_last (iter);
       iter = clutter_model_iter_next (iter))
  {
    dump_row (model, iter);
  }

  printf ("---\n");

  for (gboolean at_start = FALSE,
       iter = clutter_model_get_last_iter (model);
       iter && !at_start && !clutter_model_iter_is_last (iter);
       at_start = clutter_model_iter_is_first (iter),
       iter = clutter_model_iter_prev (iter))
  {
    dump_row (model, iter);
  }
}

static gboolean
_filter_cb (ClutterModel      *model,
            ClutterModelIter  *iter,
            const gchar       *needle)
{
  MaiPackage  *package = NULL;
  const gchar *haystack;
  gboolean     ret;

  if (!needle)
    return TRUE;

  clutter_model_iter_get (iter, 0, &package, -1);
  haystack = mai_package_get_package_name (package);
  ret = NULL != g_strstr_len (haystack, -1, needle);
  g_object_unref (package);

  return ret;
}

int
main (int     argc,
      char  **argv)
{
  ClutterModel  *model;
  MaiPackage    *package;
  const gchar   *filter = NULL;

  gtk_init (&argc, &argv);
  clutter_init (&argc, &argv);

  g_setenv ("MAI_OFFLINE", "1", FALSE);

  if (argc > 1)
    filter = argv[1];

  model = mai_object_store_new (4,
                                MAI_TYPE_PACKAGE, "package",
                                G_TYPE_STRING,    "package-name",
                                G_TYPE_STRING,    "name",
                                G_TYPE_STRING,    "version");

  package = g_object_new (MAI_TYPE_PACKAGE,
                          "package-name", "xterm",
                          "name",         "X Terminal",
                          "version",      "3.4",
                          "description",  "X terminal emulator",
                          NULL);
  clutter_model_append (model, 0, package, -1);
  g_object_unref (package), package = NULL;

  package = g_object_new (MAI_TYPE_PACKAGE,
                          "package-name", "gedit",
                          "name",         "Text Editor",
                          "version",      "2.26",
                          "description",  "Text editor for GNOME",
                          NULL);
  clutter_model_append (model, 0, package, -1);
  g_object_unref (package), package = NULL;

  clutter_model_set_filter (model,
                            (ClutterModelFilterFunc) _filter_cb,
                            (gpointer) filter,
                            NULL);

  dump (model);

  g_object_unref (model);

  return EXIT_SUCCESS;
}
